#!/usr/bin/env Rscript

options(tidyverse.quite = TRUE)
suppressPackageStartupMessages(library(tidyverse, warn.conflicts = FALSE))
library(glue, warn.conflicts = FALSE)

args <- commandArgs(trailingOnly = TRUE)

chromosome <- as.integer(args[[1]])

dosages_file       <- glue("output/dosages/chromosome-{chromosome}.dosage.txt.gz")
out_file_matched   <- glue("output/dosages/matched-chromosome-{chromosome}.dosage.txt.gz")
out_file_unmatched <- glue("output/dosages/unmatched-chromosome-{chromosome}.csv.gz")


sample_ids <- read_delim(
        "output/dosages/samples.txt",
        delim     = "\t",
        col_types = "cc",
        col_names = c("FID", "IID")
    )$IID


tbl_model_variants <- read_csv(
        "output/model-variants-lookup-table.csv.gz",
        col_types  = cols(
            chr      = col_character(),
            position = col_integer(),
            rsid     = col_character(),
            gtex_id  = col_character(),
            allele1  = col_character(),
            allele2  = col_character()
        )
    ) %>%
    filter(
        chr == glue("chr{chromosome}")
    )

tbl_dosage <- read_csv(
    glue("output/dosages/chromosome-{chromosome}.dosage.csv"),
    trim_ws    = TRUE,
    na         = ".",
    progress   = TRUE,
    col_names  = c(
        "chr",
        "rsid",
        "position",
        "allele1",
        "allele2",
        "MAF",
        sample_ids
    ),
    col_types  = cols(
        .default = col_double(),
        chr      = col_character(),
        rsid     = col_character(),
        position = col_integer(),
        allele1  = col_character(),
        allele2  = col_character()
    )
)



tbl_matched <- tbl_model_variants %>%
    inner_join(
        select(tbl_dosage, -chr, -rsid),
        by = c("position", "allele1", "allele2")
    )

write_delim(
    tbl_matched %>%
        arrange(position) %>%
        select(-rsid) %>%
        select(chr, gtex_id, position, allele1, allele2, MAF, everything()) %>%
        rename(chromosome = chr) %>%
        filter(complete.cases(.)),
    delim     = " ",
    col_names = FALSE,
    path      = out_file_matched
)



tbl_unmatched <- left_join(
        tbl_model_variants,
        select(tbl_dosage, position, allele1, allele2),
        by     = "position",
        suffix = c("..model", "..dosage")
    ) %>%
    filter(
        !(gtex_id %in% tbl_matched$gtex_id)
    )

write_csv(
    tbl_unmatched,
    path  = out_file_unmatched
)

cat(sprintf(
    "%s: %9i (%6.2f%%) model variants found for chromosome %i\n\r",
    Sys.time(),
    nrow(tbl_matched), 100*nrow(tbl_matched)/nrow(tbl_model_variants),
    chromosome
))
